@@ -7,11 +7,13 @@ import (
 	"hash/fnv"
 	"net"
 	"path/filepath"
-	"reflect"
 	"sort"
+	"strconv"
 	"strings"
 
 	"github.com/davecgh/go-spew/spew"
+	"github.com/google/go-cmp/cmp"
+	"github.com/google/go-cmp/cmp/cmpopts"
 
 	operatorv1 "github.com/openshift/api/operator/v1"
 	"github.com/openshift/cluster-ingress-operator/pkg/manifests"
@@ -30,6 +32,18 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 )
 
+const (
+	WildcardRouteAdmissionPolicy = "ROUTER_ALLOW_WILDCARD_ROUTES"
+
+	RouterLogLevelEnvName       = "ROUTER_LOG_LEVEL"
+	RouterSyslogAddressEnvName  = "ROUTER_SYSLOG_ADDRESS"
+	RouterSyslogFormatEnvName   = "ROUTER_SYSLOG_FORMAT"
+	RouterSyslogFacilityEnvName = "ROUTER_LOG_FACILITY"
+
+	RouterDisableHTTP2EnvName          = "ROUTER_DISABLE_HTTP2"
+	RouterDefaultEnableHTTP2Annotation = "ingress.operator.openshift.io/default-enable-http2"
+)
+
 // ensureRouterDeployment ensures the router deployment exists for a given
 // ingresscontroller.
 func (r *reconciler) ensureRouterDeployment(ci *operatorv1.IngressController, infraConfig *configv1.Infrastructure, ingressConfig *configv1.Ingress, apiConfig *configv1.APIServer, networkConfig *configv1.Network) (*appsv1.Deployment, error) {
@@ -71,6 +85,33 @@ func (r *reconciler) ensureRouterDeleted(ci *operatorv1.IngressController) error
 	return nil
 }
 
+// HTTP2IsEnabledByAnnotation returns true if the map m has the key
+// RouterDisableHTTP2Annotation present and true|false depending on
+// the annotation's value that is parsed by strconv.ParseBool.
+func HTTP2IsEnabledByAnnotation(m map[string]string) (bool, bool) {
+	if val, ok := m[RouterDefaultEnableHTTP2Annotation]; ok {
+		v, _ := strconv.ParseBool(val)
+		return true, v
+	}
+	return false, false
+}
+
+// HTTP2IsEnabled returns true if the ingress controller enables
+// http/2, or if the ingress config enables http/2. It will return
+// false for the case where the ingress config has been enabled but
+// the ingress controller explicitly overrides that by having the
+// annotation present (even if its value is "false").
+func HTTP2IsEnabled(ic *operatorv1.IngressController, ingressConfig *configv1.Ingress) bool {
+	controllerHasHTTP2Annotation, controllerHasHTTP2Enabled := HTTP2IsEnabledByAnnotation(ic.Annotations)
+	_, configHasHTTP2Enabled := HTTP2IsEnabledByAnnotation(ingressConfig.Annotations)
+
+	if controllerHasHTTP2Annotation {
+		return controllerHasHTTP2Enabled
+	}
+
+	return configHasHTTP2Enabled
+}
+
 // desiredRouterDeployment returns the desired router deployment.
 func desiredRouterDeployment(ci *operatorv1.IngressController, ingressControllerImage string, infraConfig *configv1.Infrastructure, ingressConfig *configv1.Ingress, apiConfig *configv1.APIServer, networkConfig *configv1.Network) (*appsv1.Deployment, error) {
 	deployment := manifests.RouterDeployment()
@@ -94,6 +135,12 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 	volumes := deployment.Spec.Template.Spec.Volumes
 	routerVolumeMounts := deployment.Spec.Template.Spec.Containers[0].VolumeMounts
 
+	var desiredReplicas int32 = 2
+	if ci.Spec.Replicas != nil {
+		desiredReplicas = *ci.Spec.Replicas
+	}
+	deployment.Spec.Replicas = &desiredReplicas
+
 	needDeploymentHash := false
 	switch ci.Status.EndpointPublishingStrategy.Type {
 	case operatorv1.HostNetworkStrategyType:
@@ -124,13 +171,21 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		// the deployment strategy: During a rolling update, we want the
 		// deployment controller to scale up the new replica set first
 		// and scale down the old replica set once the new replica is
-		// ready.  Thus set max unavailable to 0 and surge to 1.
+		// ready.  Thus set max unavailable to 50% (if replicas < 4) or
+		// 25% (if replicas >= 4) and surge to 25%.  Note that the
+		// deployment controller rounds surge up and max unavailable
+		// down.
+
+		maxUnavailable := "50%"
+		if desiredReplicas >= 4 {
+			maxUnavailable = "25%"
+		}
 		pointerTo := func(ios intstr.IntOrString) *intstr.IntOrString { return &ios }
 		deployment.Spec.Strategy = appsv1.DeploymentStrategy{
 			Type: appsv1.RollingUpdateDeploymentStrategyType,
 			RollingUpdate: &appsv1.RollingUpdateDeployment{
-				MaxUnavailable: pointerTo(intstr.FromInt(0)),
-				MaxSurge:       pointerTo(intstr.FromInt(1)),
+				MaxUnavailable: pointerTo(intstr.FromString(maxUnavailable)),
+				MaxSurge:       pointerTo(intstr.FromString("25%")),
 			},
 		}
 
@@ -292,12 +347,6 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		})
 	}
 
-	var desiredReplicas int32 = 2
-	if ci.Spec.Replicas != nil {
-		desiredReplicas = *ci.Spec.Replicas
-	}
-	deployment.Spec.Replicas = &desiredReplicas
-
 	if ci.Spec.RouteSelector != nil {
 		routeSelector, err := metav1.LabelSelectorAsSelector(ci.Spec.RouteSelector)
 		if err != nil {
@@ -325,7 +374,11 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 	secretName := controller.RouterEffectiveDefaultCertificateSecretName(ci, deployment.Namespace)
 	deployment.Spec.Template.Spec.Volumes[0].Secret.SecretName = secretName.Name
 
-	if enabled, level := ExtraLoggingEnabled(ci, ingressConfig); enabled {
+	accessLogging := accessLoggingForIngressController(ci)
+	switch {
+	case accessLogging == nil:
+		// No additional configuration is needed.
+	case accessLogging.Destination.Type == operatorv1.ContainerLoggingDestinationType:
 		rsyslogConfigVolume := corev1.Volume{
 			Name: "rsyslog-config",
 			VolumeSource: corev1.VolumeSource{
@@ -360,7 +413,7 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		socketPath := filepath.Join(rsyslogSocketVolumeMount.MountPath, "rsyslog.sock")
 
 		syslogContainer := corev1.Container{
-			Name: "syslog",
+			Name: operatorv1.ContainerLoggingSidecarContainerName,
 			// The ingresscontroller image has rsyslog built in.
 			Image: ingressControllerImage,
 			Command: []string{
@@ -384,12 +437,29 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		}
 
 		env = append(env,
-			corev1.EnvVar{Name: "ROUTER_SYSLOG_ADDRESS", Value: socketPath},
-			corev1.EnvVar{Name: "ROUTER_LOG_LEVEL", Value: level},
+			corev1.EnvVar{Name: RouterSyslogAddressEnvName, Value: socketPath},
+			corev1.EnvVar{Name: RouterLogLevelEnvName, Value: "debug"},
 		)
+		if len(accessLogging.HttpLogFormat) > 0 {
+			env = append(env, corev1.EnvVar{Name: RouterSyslogFormatEnvName, Value: fmt.Sprintf("%q", accessLogging.HttpLogFormat)})
+		}
 		volumes = append(volumes, rsyslogConfigVolume, rsyslogSocketVolume)
 		routerVolumeMounts = append(routerVolumeMounts, rsyslogSocketVolumeMount)
 		deployment.Spec.Template.Spec.Containers = append(deployment.Spec.Template.Spec.Containers, syslogContainer)
+	case accessLogging.Destination.Type == operatorv1.SyslogLoggingDestinationType:
+		if len(accessLogging.Destination.Syslog.Facility) > 0 {
+			env = append(env, corev1.EnvVar{Name: RouterSyslogFacilityEnvName, Value: accessLogging.Destination.Syslog.Facility})
+		}
+		address := accessLogging.Destination.Syslog.Address
+		port := accessLogging.Destination.Syslog.Port
+		endpoint := net.JoinHostPort(address, fmt.Sprintf("%d", port))
+		env = append(env,
+			corev1.EnvVar{Name: RouterLogLevelEnvName, Value: "debug"},
+			corev1.EnvVar{Name: RouterSyslogAddressEnvName, Value: endpoint},
+		)
+		if len(accessLogging.HttpLogFormat) > 0 {
+			env = append(env, corev1.EnvVar{Name: RouterSyslogFormatEnvName, Value: fmt.Sprintf("%q", accessLogging.HttpLogFormat)})
+		}
 	}
 
 	tlsProfileSpec := tlsProfileSpecForIngressController(ci, apiConfig)
@@ -436,9 +506,15 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 
 	routeAdmission := operatorv1.RouteAdmissionPolicy{
 		NamespaceOwnership: operatorv1.StrictNamespaceOwnershipCheck,
+		WildcardPolicy:     operatorv1.WildcardPolicyDisallowed,
 	}
-	if admission := ci.Spec.RouteAdmission; admission != nil && len(admission.NamespaceOwnership) > 0 {
-		routeAdmission.NamespaceOwnership = ci.Spec.RouteAdmission.NamespaceOwnership
+	if admission := ci.Spec.RouteAdmission; admission != nil {
+		if len(admission.NamespaceOwnership) > 0 {
+			routeAdmission.NamespaceOwnership = admission.NamespaceOwnership
+		}
+		if len(admission.WildcardPolicy) > 0 {
+			routeAdmission.WildcardPolicy = admission.WildcardPolicy
+		}
 	}
 	switch routeAdmission.NamespaceOwnership {
 	case operatorv1.StrictNamespaceOwnershipCheck:
@@ -446,6 +522,18 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 	case operatorv1.InterNamespaceAllowedOwnershipCheck:
 		env = append(env, corev1.EnvVar{Name: "ROUTER_DISABLE_NAMESPACE_OWNERSHIP_CHECK", Value: "true"})
 	}
+	switch routeAdmission.WildcardPolicy {
+	case operatorv1.WildcardPolicyAllowed:
+		env = append(env, corev1.EnvVar{Name: WildcardRouteAdmissionPolicy, Value: "true"})
+	default:
+		env = append(env, corev1.EnvVar{Name: WildcardRouteAdmissionPolicy, Value: "false"})
+	}
+
+	if HTTP2IsEnabled(ci, ingressConfig) {
+		env = append(env, corev1.EnvVar{Name: RouterDisableHTTP2EnvName, Value: "false"})
+	} else {
+		env = append(env, corev1.EnvVar{Name: RouterDisableHTTP2EnvName, Value: "true"})
+	}
 
 	deployment.Spec.Template.Spec.Volumes = volumes
 	deployment.Spec.Template.Spec.Containers[0].VolumeMounts = routerVolumeMounts
@@ -465,6 +553,41 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 	return deployment, nil
 }
 
+// accessLoggingForIngressController returns an AccessLogging value for the
+// given ingresscontroller, or nil if the ingresscontroller does not specify
+// a valid access logging configuration.
+func accessLoggingForIngressController(ic *operatorv1.IngressController) *operatorv1.AccessLogging {
+	if ic.Spec.Logging == nil || ic.Spec.Logging.Access == nil {
+		return nil
+	}
+
+	switch ic.Spec.Logging.Access.Destination.Type {
+	case operatorv1.ContainerLoggingDestinationType:
+		return &operatorv1.AccessLogging{
+			Destination: operatorv1.LoggingDestination{
+				Type:      operatorv1.ContainerLoggingDestinationType,
+				Container: &operatorv1.ContainerLoggingDestinationParameters{},
+			},
+			HttpLogFormat: ic.Spec.Logging.Access.HttpLogFormat,
+		}
+	case operatorv1.SyslogLoggingDestinationType:
+		if ic.Spec.Logging.Access.Destination.Syslog != nil {
+			return &operatorv1.AccessLogging{
+				Destination: operatorv1.LoggingDestination{
+					Type: operatorv1.SyslogLoggingDestinationType,
+					Syslog: &operatorv1.SyslogLoggingDestinationParameters{
+						Address:  ic.Spec.Logging.Access.Destination.Syslog.Address,
+						Port:     ic.Spec.Logging.Access.Destination.Syslog.Port,
+						Facility: ic.Spec.Logging.Access.Destination.Syslog.Facility,
+					},
+				},
+				HttpLogFormat: ic.Spec.Logging.Access.HttpLogFormat,
+			}
+		}
+	}
+	return nil
+}
+
 // inferTLSProfileSpecFromDeployment examines the given deployment's pod
 // template spec and reconstructs a TLS profile spec based on that pod spec.
 func inferTLSProfileSpecFromDeployment(deployment *appsv1.Deployment) *configv1.TLSProfileSpec {
@@ -640,9 +763,8 @@ func hashableDeployment(deployment *appsv1.Deployment, onlyTemplate bool) *appsv
 			Image:           container.Image,
 			ImagePullPolicy: container.ImagePullPolicy,
 			Name:            container.Name,
-			LivenessProbe:   container.LivenessProbe,
-			ReadinessProbe:  container.ReadinessProbe,
-			VolumeMounts:    container.VolumeMounts,
+			LivenessProbe:   hashableProbe(container.LivenessProbe),
+			ReadinessProbe:  hashableProbe(container.ReadinessProbe),
 		}
 	}
 	sort.Slice(containers, func(i, j int) bool {
@@ -687,6 +809,43 @@ func hashableDeployment(deployment *appsv1.Deployment, onlyTemplate bool) *appsv
 	return &hashableDeployment
 }
 
+// hashableProbe returns a copy of the given probe with exactly the fields that
+// should be used for computing a deployment's hash copied over.  Fields that
+// should be ignored, or that have explicit values that are equal to their
+// respective default values, will be zeroed.
+func hashableProbe(probe *corev1.Probe) *corev1.Probe {
+	if probe == nil {
+		return nil
+	}
+
+	var hashableProbe corev1.Probe
+
+	if probe.Handler.HTTPGet != nil {
+		hashableProbe.Handler.HTTPGet = &corev1.HTTPGetAction{
+			Path: probe.Handler.HTTPGet.Path,
+			Port: probe.Handler.HTTPGet.Port,
+			Host: probe.Handler.HTTPGet.Host,
+		}
+		if probe.Handler.HTTPGet.Scheme != "HTTP" {
+			hashableProbe.Handler.HTTPGet.Scheme = probe.Handler.HTTPGet.Scheme
+		}
+	}
+	if probe.TimeoutSeconds != int32(1) {
+		hashableProbe.TimeoutSeconds = probe.TimeoutSeconds
+	}
+	if probe.PeriodSeconds != int32(10) {
+		hashableProbe.PeriodSeconds = probe.PeriodSeconds
+	}
+	if probe.SuccessThreshold != int32(1) {
+		hashableProbe.SuccessThreshold = probe.SuccessThreshold
+	}
+	if probe.FailureThreshold != int32(3) {
+		hashableProbe.FailureThreshold = probe.FailureThreshold
+	}
+
+	return &hashableProbe
+}
+
 // currentRouterDeployment returns the current router deployment.
 func (r *reconciler) currentRouterDeployment(ci *operatorv1.IngressController) (*appsv1.Deployment, error) {
 	deployment := &appsv1.Deployment{}
@@ -715,48 +874,12 @@ func (r *reconciler) updateRouterDeployment(current, desired *appsv1.Deployment)
 		return nil
 	}
 
-	// It may be impossible to transition from the current state to the
-	// desired state if the current state has an affinity policy with a pod
-	// anti-affinity rule that prohibits colocation of pods belonging to the
-	// same ingresscontroller and the desired state requires using surge for
-	// rolling updates.  Thus, if the current deployment has an old affinity
-	// policy, we update the deployment in two steps:
-	//
-	// 1. Update the affinity policy (which should enable colocation of pods
-	// with different deployment hashes).
-	//
-	// 2. Once that update is rolled out, update the deployment strategy to
-	// use surge.
-	//
-	// TODO: This migration was introduced in OpenShift 4.3 and can be
-	// removed in OpenShift 4.4.
-	switch {
-	case reflect.DeepEqual(current.Spec.Strategy, updated.Spec.Strategy):
-		// Migration is complete (or was never needed), so continue with
-		// the update as usual.
-	case reflect.DeepEqual(current.Spec.Template.Spec.Affinity, updated.Spec.Template.Spec.Affinity):
-		// Migration has been started but not completed.
-		if current.Status.UpdatedReplicas != current.Status.Replicas {
-			// We cannot continue to the next step till the rollout
-			// is complete, so log and return.
-			log.Info("waiting for rollout of new affinity policy to complete", "namespace", current.Namespace, "name", current.Name)
-			return nil
-		}
-
-		// Update of the affinity policy is complete; now update the
-		// deployment strategy by continuing with the update as normal.
-		log.Info("migrating router deployment to new deployment strategy", "namespace", current.Namespace, "name", current.Name)
-	default:
-		// Initiate migration by updating affinity but keeping the old
-		// strategy.
-		updated.Spec.Strategy = current.Spec.Strategy
-		log.Info("migrating router deployment to new affinity policy", "namespace", current.Namespace, "name", current.Name)
-	}
-
+	// Diff before updating because the client may mutate the object.
+	diff := cmp.Diff(current, updated, cmpopts.EquateEmpty())
 	if err := r.client.Update(context.TODO(), updated); err != nil {
 		return fmt.Errorf("failed to update router deployment %s/%s: %v", updated.Namespace, updated.Name, err)
 	}
-	log.Info("updated router deployment", "namespace", updated.Namespace, "name", updated.Name)
+	log.Info("updated router deployment", "namespace", updated.Namespace, "name", updated.Name, "diff", diff)
 	return nil
 }
 
@@ -802,6 +925,8 @@ func deploymentConfigChanged(current, expected *appsv1.Deployment) (bool, *appsv
 	updated.Spec.Template.Spec.NodeSelector = expected.Spec.Template.Spec.NodeSelector
 	updated.Spec.Template.Spec.Containers[0].Env = expected.Spec.Template.Spec.Containers[0].Env
 	updated.Spec.Template.Spec.Containers[0].Image = expected.Spec.Template.Spec.Containers[0].Image
+	updated.Spec.Template.Spec.Containers[0].LivenessProbe = expected.Spec.Template.Spec.Containers[0].LivenessProbe
+	updated.Spec.Template.Spec.Containers[0].ReadinessProbe = expected.Spec.Template.Spec.Containers[0].ReadinessProbe
 	updated.Spec.Template.Spec.Containers[0].VolumeMounts = expected.Spec.Template.Spec.Containers[0].VolumeMounts
 	updated.Spec.Template.Spec.Tolerations = expected.Spec.Template.Spec.Tolerations
 	updated.Spec.Template.Spec.Affinity = expected.Spec.Template.Spec.Affinity
